@@ -1,20 +1,20 @@
 /* Replacements for routines missing on some systems.
    Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
 
-This file is part of Wget.
+This file is part of GNU Wget.
 
-This program is free software; you can redistribute it and/or modify
+GNU Wget is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
+GNU Wget is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
+along with Wget; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
@@ -26,7 +26,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #else
 # include <strings.h>
 #endif /* HAVE_STRING_H */
-#include <ctype.h>
 
 #include <sys/types.h>
 #ifdef HAVE_UNISTD_H
@@ -657,9 +656,9 @@ strptime_internal (buf, format, tm, decided)
     {
       /* A white space in the format string matches 0 more or white
 	 space in the input string.  */
-      if (isspace (*fmt))
+      if (ISSPACE (*fmt))
 	{
-	  while (isspace (*rp))
+	  while (ISSPACE (*rp))
 	    ++rp;
 	  ++fmt;
 	  continue;
@@ -851,7 +850,7 @@ strptime_internal (buf, format, tm, decided)
 	case 'n':
 	case 't':
 	  /* Match any white space.  */
-	  while (isspace (*rp))
+	  while (ISSPACE (*rp))
 	    ++rp;
 	  break;
 	case 'p':
