@@ -1,19 +1,21 @@
 /* Declarations for FTP support.
-   Copyright (C) 1995, 1996, 1997 Free Software Foundation, Inc.
+   Copyright (C) 1995, 1996, 1997, 2000 Free Software Foundation, Inc.
 
-   This program is free software; you can redistribute it and/or modify
-   it under the terms of the GNU General Public License as published by
-   the Free Software Foundation; either version 2 of the License, or
-   (at your option) any later version.
+This file is part of GNU Wget.
 
-   This program is distributed in the hope that it will be useful,
-   but WITHOUT ANY WARRANTY; without even the implied warranty of
-   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
-   GNU General Public License for more details.
+GNU Wget is free software; you can redistribute it and/or modify
+it under the terms of the GNU General Public License as published by
+the Free Software Foundation; either version 2 of the License, or
+(at your option) any later version.
 
-   You should have received a copy of the GNU General Public License
-   along with this program; if not, write to the Free Software
-   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
+GNU Wget is distributed in the hope that it will be useful,
+but WITHOUT ANY WARRANTY; without even the implied warranty of
+MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
+GNU General Public License for more details.
+
+You should have received a copy of the GNU General Public License
+along with Wget; if not, write to the Free Software
+Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #ifndef FTP_H
 #define FTP_H
@@ -21,6 +23,16 @@
 /* Need it for struct rbuf.  */
 #include "rbuf.h"
 
+/* System types. */
+enum stype
+{
+  ST_UNIX,
+  ST_VMS,
+  ST_WINNT,
+  ST_MACOS,
+  ST_OTHER
+};
+  
 uerr_t ftp_response PARAMS ((struct rbuf *, char **));
 uerr_t ftp_login PARAMS ((struct rbuf *, const char *, const char *));
 uerr_t ftp_port PARAMS ((struct rbuf *));
@@ -30,6 +42,8 @@ uerr_t ftp_cwd PARAMS ((struct rbuf *, const char *));
 uerr_t ftp_retr PARAMS ((struct rbuf *, const char *));
 uerr_t ftp_rest PARAMS ((struct rbuf *, long));
 uerr_t ftp_list PARAMS ((struct rbuf *, const char *));
+uerr_t ftp_syst PARAMS ((struct rbuf *, enum stype *));
+uerr_t ftp_pwd PARAMS ((struct rbuf *, char **));
 
 struct urlinfo;
 
@@ -63,16 +77,18 @@ struct fileinfo
 };
 
 /* Commands for FTP functions.  */
-enum command
+enum wget_ftp_command
 {
   DO_LOGIN      = 0x0001,	/* Connect and login to the server.  */
   DO_CWD        = 0x0002,	/* Change current directory.  */
   DO_RETR       = 0x0004,	/* Retrieve the file.  */
   DO_LIST       = 0x0008,	/* Retrieve the directory list.  */
-  LEAVE_PENDING = 0x0010	/* Do not close the socket.  */
+  LEAVE_PENDING = 0x0010,	/* Do not close the socket.  */
+  NO_TRUNCATE	= 0x0020	/* Don't truncate the file if REST
+				   malfunctions. */
 };
 
-enum fstatus
+enum wget_ftp_fstatus
 {
   NOTHING       = 0x0000,	/* Nothing done yet.  */
   ON_YOUR_OWN   = 0x0001,	/* The ftp_loop_internal sets the
@@ -87,9 +103,13 @@ typedef struct
   int cmd;			/* command code */
   struct rbuf rbuf;		/* control connection buffer */
   long dltime;			/* time of the download */
+  enum stype rs;		/* remote system reported by ftp server */ 
+  char *id;			/* initial directory */
 } ccon;
 
-struct fileinfo *ftp_parse_ls PARAMS ((const char *));
+struct fileinfo *ftp_parse_ls PARAMS ((const char *, enum stype));
 uerr_t ftp_loop PARAMS ((struct urlinfo *, int *));
 
+uerr_t ftp_index (const char *, struct urlinfo *, struct fileinfo *);
+
 #endif /* FTP_H */
