@@ -1,20 +1,20 @@
 /* Messages logging.
    Copyright (C) 1998, 2000 Free Software Foundation, Inc.
 
-This file is part of Wget.
+This file is part of GNU Wget.
 
-This program is free software; you can redistribute it and/or modify
+GNU Wget is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
+GNU Wget is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
+along with Wget; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 #include <config.h>
@@ -112,7 +112,7 @@ free_log_line (int num)
   struct log_ln *ln = log_lines + num;
   if (ln->malloced_line)
     {
-      free (ln->malloced_line);
+      xfree (ln->malloced_line);
       ln->malloced_line = NULL;
     }
   ln->content = NULL;
@@ -346,7 +346,7 @@ logvprintf (enum log_options o, const char *fmt, va_list args)
       saved_append (write_ptr);
       fputs (write_ptr, logfp);
       if (bigmsg)
-	free (bigmsg);
+	xfree (bigmsg);
     }
   if (!opt.no_flush)
     fflush (logfp);
@@ -524,7 +524,7 @@ redirect_output (const char *messij)
       logfp = stdin;
     }
   fprintf (stderr, messij, logfile);
-  free (logfile);
+  xfree (logfile);
   /* Dump the previous screenful of output to LOGFILE.  */
   log_dump ();
   save_log_p = 0;
