@@ -1,23 +1,23 @@
 /* mswindows.c -- Windows-specific support
    Copyright (C) 1995, 1996, 1997, 1998  Free Software Foundation, Inc.
 
-This file is part of Wget.
+This file is part of GNU Wget.
 
-This program is free software; you can redistribute it and/or modify
+GNU Wget is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
 the Free Software Foundation; either version 2 of the License, or
 (at your option) any later version.
 
-This program is distributed in the hope that it will be useful,
+GNU Wget is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
+along with Wget; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
-/* #### Someone document these functions!  */
+/* #### Someone please document what these functions do!  */
 
 #include <config.h>
 
@@ -35,8 +35,6 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 extern int errno;
 #endif
 
-char *argv0;
-
 /* Defined in log.c.  */
 void redirect_output (const char *);
 
@@ -47,11 +45,7 @@ static int windows_nt_p;
 unsigned int
 sleep (unsigned seconds)
 {
-  Sleep (1000 * seconds);
-  /* Unix sleep() is interruptible.  To make it semi-usable, it
-     returns a value that says how much it "really" slept, or some
-     junk like that.  Ignore it.  */
-  return 0U;
+  return SleepEx (1000 * seconds, TRUE) ? 0U : 1000 * seconds;
 }
 
 static char *
@@ -69,39 +63,11 @@ read_registry (HKEY hkey, char *subkey, char *valuename, char *buf, int *len)
   return buf;
 }
 
-char *
-pwd_cuserid (char *where)
-{
-  char buf[32], *ptr;
-  int len = sizeof (buf);
-  if (GetUserName (buf, (LPDWORD) &len) == TRUE)
-    {
-      ;
-    }
-  else if (!!(ptr = getenv ("USERNAME")))
-    {
-      strcpy (buf, ptr);
-    }
-  else if (!read_registry (HKEY_LOCAL_MACHINE, "Network\\Logon",
-			  "username", buf, &len))
-    {
-      return NULL;
-    }
-  if (where)
-    {
-      strncpy (where, buf, len);
-      return where;
-    }
-  return xstrdup (buf);
-}
-
 void
 windows_main_junk (int *argc, char **argv, char **exec_name)
 {
   char *p;
 
-  argv0 = argv[0];
-
   /* Remove .EXE from filename if it has one.  */
   *exec_name = xstrdup (*exec_name);
   p = strrchr (*exec_name, '.');
@@ -176,10 +142,8 @@ ws_changetitle (char *url, int nurl)
   if (!nurl)
     return;
 
-  title_buf = (char *)xmalloc (strlen (url) + 20);
+  title_buf = (char *)alloca (strlen (url) + 20);
   sprintf (title_buf, "Wget %s%s", url, nurl == 1 ? "" : " ...");
-  /* #### What are the semantics of SetConsoleTitle?  Will it free the
-     given memory later?  */
   SetConsoleTitle (title_buf);
 }
 
@@ -187,34 +151,26 @@ char *
 ws_mypath (void)
 {
   static char *wspathsave;
-  char *buffer;
-  int rrr;
+  char buffer[MAX_PATH];
   char *ptr;
 
   if (wspathsave)
     {
       return wspathsave;
     }
-  ptr = strrchr (argv0, '\\');
+
+  GetModuleFileName (NULL, buffer, MAX_PATH);
+
+  ptr = strrchr (buffer, '\\');
   if (ptr)
     {
       *(ptr + 1) = '\0';
-      wspathsave = (char*) xmalloc (strlen(argv0)+1);
-      strcpy (wspathsave, argv0);
-      return wspathsave;
-    }
-  buffer = (char*) xmalloc (256);
-  rrr = SearchPath (NULL, argv0, strchr (argv0, '.') ? NULL : ".EXE",
-		    256, buffer, &ptr);
-  if (rrr && rrr <= 256)
-    {
-      *ptr = '\0';
-      wspathsave = (char*) xmalloc (strlen(buffer)+1);
+      wspathsave = (char*) xmalloc (strlen (buffer) + 1);
       strcpy (wspathsave, buffer);
-      return wspathsave;
     }
-  free (buffer);
-  return NULL;
+  else
+    wspathsave = NULL;
+  return wspathsave;
 }
 
 void
@@ -261,8 +217,7 @@ ws_startup (void)
       exit (1);
     }
 
-  if (LOBYTE (requested) < 1 || (LOBYTE (requested) == 1 &&
-				HIBYTE (requested) < 1))
+  if (data.wVersion < requested)
     {
       fprintf (stderr, _("%s: Couldn't find usable socket driver.\n"),
 	       exec_name);
