@@ -1,20 +1,20 @@
 /* Dirty system-dependent hacks.
    Copyright (C) 1995, 1996, 1997, 1998, 2000 Free Software Foundation, Inc.
 
-This file is part of Wget.
+This file is part of GNU Wget.
 
-This program is free software; you can redistribute it and/or modify
+GNU Wget is free software; you can redistribute it and/or modify
 it under the terms of the GNU General Public License as published by
-the Free Software Foundation; either version 2 of the License, or
-(at your option) any later version.
+the Free Software Foundation; either version 2 of the License, or (at
+your option) any later version.
 
-This program is distributed in the hope that it will be useful,
+GNU Wget is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.
 
 You should have received a copy of the GNU General Public License
-along with this program; if not, write to the Free Software
+along with Wget; if not, write to the Free Software
 Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 
 /* This file is included by wget.h.  Random .c files need not include
@@ -45,6 +45,20 @@ Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.  */
 #include <mswindows.h>
 #endif /* WINDOWS */
 
+/* Watcom-specific stuff.  In practice this is probably specific to
+   Windows, although Watcom exists under other OS's too.  For that
+   reason, we keep this here.  */
+
+#ifdef __WATCOMC__
+/* Watcom has its own alloca() defined in malloc.h malloc.h needs to
+   be included in the sources to prevent 'undefined external' errors
+   at the link phase. */
+# include <malloc.h>
+/* io.h defines unlink() and chmod().  We put this here because it's
+   way too obscure to require all the .c files to observe.  */
+# include <io.h>
+#endif /* __WATCOMC__ */
+
 /* Needed for compilation under OS/2: */
 #ifdef __EMX__
 #ifndef S_ISLNK
@@ -97,40 +111,32 @@ do {						\
   DEBUGP (("Closing fd %d\n", x));		\
 } while (0)
 
-/* Define a "very long" type useful for storing large non-negative
-   integers, e.g. the total number of bytes downloaded.  This needs to
-   be an integral type at least 64 bits wide.  On the machines where
-   `long' is 64-bit, we use long.  Otherwise, we check whether `long
-   long' is available and if yes, use that.  Otherwise, we give up and
-   just use `long'.  */
+/* Define a large ("very long") type useful for storing large
+   non-negative quantities that exceed sizes of normal download, such
+   as the *total* number of bytes downloaded.  To fit today's needs,
+   this needs to be an integral type at least 64 bits wide.  On the
+   machines where `long' is 64-bit, we use long.  Otherwise, we check
+   whether `long long' is available and if yes, use that.  If long
+   long is unavailable, we give up and just use `long'.
+
+   This check could be smarter and moved to configure, which could
+   check for a bunch of non-standard types such as uint64_t.  But I
+   don't see the need for it -- the current test will work on all
+   modern architectures, and if it fails, nothing bad happens, we just
+   end up with long.
+
+   Note: you cannot use VERY_LONG_TYPE along with printf().  When you
+   need to print it, use very_long_to_string().  */
+
 #if (SIZEOF_LONG >= 8) || !defined(HAVE_LONG_LONG)
+/* either long is "big enough", or long long is unavailable which
+   leaves long as the only choice. */ 
 # define VERY_LONG_TYPE   unsigned long
-# define VERY_LONG_FORMAT "%lu"
-#else  /* long is smaller than 8 bytes, but long long is available. */
+#else  /* use long long */
+/* long is smaller than 8 bytes, but long long is available. */
 # define VERY_LONG_TYPE   unsigned long long
-# define VERY_LONG_FORMAT "%llu"
 #endif /* use long long */
 
-/* OK, now define a decent interface to ctype macros.  The regular
-   ones misfire when you feed them chars > 127, as they understand
-   them as "negative", which results in out-of-bound access at
-   table-lookup, yielding random results.  This is, of course, totally
-   bogus.  One way to "solve" this is to use `unsigned char'
-   everywhere, but it is nearly impossible to do that cleanly, because
-   all of the library functions and system calls accept `char'.
-
-   Thus we define our wrapper macros which simply cast the argument to
-   unsigned char before passing it to the <ctype.h> macro.  These
-   versions are used consistently across the code.  */
-#define ISASCII(x)  isascii ((unsigned char)(x))
-#define ISALPHA(x)  isalpha ((unsigned char)(x))
-#define ISALNUM(x)  isalnum ((unsigned char)(x))
-#define ISSPACE(x)  isspace ((unsigned char)(x))
-#define ISDIGIT(x)  isdigit ((unsigned char)(x))
-#define ISXDIGIT(x) isxdigit ((unsigned char)(x))
-#define TOUPPER(x)  toupper ((unsigned char)(x))
-#define TOLOWER(x)  tolower ((unsigned char)(x))
-
 /* Defined in cmpt.c: */
 #ifndef HAVE_STRERROR
 char *strerror ();
